@@ -12,38 +12,9 @@ extern time_t opt_timestamp;
 extern gint opt_verbose;
 extern GSList *pregex_list;
 
-#if 0
-/* xattr.c */
-uid_t read_attr_uid(char *path, uid_t u);
-gid_t read_attr_gid(char *path, gid_t g);
-#endif
-
-static struct r_entry *
-entry_dup(struct r_entry *f)
-{
-        struct r_entry *g;
-        g = g_malloc(sizeof(struct r_entry));
-
-        g->f_name       = g_strdup(f->f_name);
-        g->f_name_size  = f->f_name_size;
-        g->f_lnk	= f->f_lnk;
-        g->f_uid        = f->f_uid;
-        g->f_gid        = f->f_gid;
-        g->f_mode       = f->f_mode;
-	g->f_ctime      = f->f_ctime;
-	g->f_size       = f->f_size;
-	g->f_dev        = f->f_dev;
-	g->f_rdev       = f->f_rdev;
-	g->f_ino        = f->f_ino;
-        return g;
-}
-
-static void
-entry_free(struct r_entry *f)
-{
-	g_free(f->f_name);
-	g_free(f);
-}
+/* common.c */
+struct r_entry * entry_dup(struct r_entry *f);
+void entry_free(struct r_entry *f);
 
 /**
  * prepend path leading up to backup directory to the tree
@@ -62,13 +33,13 @@ dir_prepend(GTree *t, char *path)
 	len   = strlen(path);
 
 	/* add closing / */
-	if (path2[len - 1] != DIR_SEP) {
+	if (path2[len - 1] != '/') {
 		path2 = g_realloc(path2, len + 2);
-		path2[len] = DIR_SEP;
+		path2[len] = '/';
 		path2[len + 1] = '\0';
 	}
 
-	for (p = path2 + 1; (c = strchr(p, DIR_SEP)); p++) {
+	for (p = path2 + 1; (c = strchr(p, '/')); p++) {
 		*c = '\0';
 		if (lstat(path2, &s) != 0) {
 			msg(_("Could not stat path `%s\': %s"), path2, strerror(errno));
@@ -91,7 +62,7 @@ dir_prepend(GTree *t, char *path)
 			e = *(sym_link(&e, NULL));
 
 		g_tree_insert(t, (gpointer) entry_dup(&e), VALUE);
-		*c = DIR_SEP;
+		*c = '/';
 		p = c++;
 	}
 	g_free(path2);
@@ -148,11 +119,11 @@ dir_crawl(GTree *t, GHashTable *linkhash, char *path)
 				!strcmp(dent->d_name, ".."))
 			continue;
 
-		if (strcmp(path, DIR_SEP_STR) == 0)  {
-			curpath = g_strdup_printf("%c%s", DIR_SEP, dent->d_name);
+		if (strcmp(path, "/") == 0)  {
+			curpath = g_strdup_printf("/%s", dent->d_name);
 			curpath_len = strlen(curpath);
 		} else {
-			curpath = g_strdup_printf("%s%c%s", path, DIR_SEP, dent->d_name);
+			curpath = g_strdup_printf("%s/%s", path, dent->d_name);
 			curpath_len = strlen(curpath);
 		}
 
