@@ -6,23 +6,22 @@
  */
 
 #include "rdup.h"
+#include "protocol.h"
 #include <pcre.h>
 
 extern gboolean opt_null;
 extern gboolean opt_removed;
 extern gboolean opt_modified;
 extern gboolean opt_skip;
-extern gboolean opt_local;
 extern gint opt_verbose;
 extern char *opt_format;
-extern char qstr[];
 extern time_t opt_timestamp;
 extern size_t opt_size;
 extern sig_atomic_t sig;
 extern GList *list;
 
 /* sha1.c */
-int sha1_stream(FILE *stream, void *resblock);
+int sha1_stream(FILE *stream, void *digest);
 
 /*
  * calculates a files sha1 sum
@@ -53,59 +52,43 @@ sha1(FILE *fp, char *filename)
  * cat the files' contents
  */
 static gboolean
-cat(FILE *fp, char *filename, off_t f_size)
+cat(FILE *fp, char *filename)
 {
 	char buf[BUFSIZE + 1];
 	FILE *file;
 	size_t i;
-	size_t t;
-	size_t missing;
+	gboolean nullblock = FALSE;
 
 	if ((file = fopen(filename, "r")) == NULL) {
 		msg(_("Could not open '%s\': %s"), filename, strerror(errno));
 		return FALSE;
 	}
 
-	t = 0;
 	while (!feof(file) && (!ferror(file))) {
 		if (sig != 0) {
 			fclose(file);
 			signal_abort(sig);
 		}
 		i = fread(buf, sizeof(char), BUFSIZE, file);
-		t += i;
-		if (t > (size_t) f_size) {
-			/* the file has grown. Break off the write!! */
-			msg(_("File grown larger (%zd) than original file size (%zd) , cutting off: `%s\'"), t, (size_t) f_size, filename);
-			/* what's missing and what is read in the previous read */
-			missing = t - i;
-			missing = (size_t) f_size - missing;
-			if (missing > 0) {
-				/* write the missing bytes till f_size */
-				if (fwrite(buf, sizeof(char), missing, fp) != missing) {
-					msg(_("Write failure `%s\': %s"), filename, strerror(errno));
-					fclose(file);
-					return FALSE;
-				}
-			}
-			fclose(file);
-			return TRUE;
-		}
-
-		if (fwrite(buf, sizeof(char), i, fp) != i) {
+		if (block_out_header(fp, i, -1) == -1 ||
+				block_out(fp, i, buf, -1)) {
 			msg(_("Write failure `%s\': %s"), filename, strerror(errno));
 			fclose(file);
 			return FALSE;
 		}
-	}
-	fclose(file);
 
-	/* file has shrunken! Fill the rest with NULLs, this works
-	 * but is slow! */
-	if (t < (size_t) f_size) {
-		msg(_("File has shrunk, filling with NULLs: `%s\'"), filename);
-		for(i = t; i < (size_t) f_size; i++) {
-			fputc('\0', fp);
+		/* there is no diff between 0 bytes block and a ending block */
+		if (i == 0)
+			nullblock = TRUE;
+
+	}
+
+	fclose(file);
+	if (!nullblock) {
+		if (block_out_header(fp, 0, -1) == -1) {
+			msg(_("Write failure `%s\': %s"), filename, strerror(errno));
+			fclose(file);
+			return FALSE;
 		}
 	}
 	return TRUE;
@@ -118,7 +101,7 @@ static void
 entry_cat_data(FILE *fp, struct r_entry *e)
 {
 	if (S_ISREG(e->f_mode) && e->f_lnk == 0) {
-		if (!cat(fp, e->f_name, e->f_size)) {
+		if (!cat(fp, e->f_name)) {
 			exit(EXIT_FAILURE);
 		}
 		return;
@@ -180,7 +163,7 @@ entry_print_data(FILE *out, char n, struct r_entry *e)
 			 * filesize has the length what we should print */
 			if (S_ISLNK(e->f_mode) || e->f_lnk == 1) {
 				if (fwrite(e->f_name, e->f_size, sizeof(char), out) != (size_t)e->f_size) {
-					msg(_("Write failure: %s"), strerror(errno));
+					msg(_("Write failure `%s\': %s"), e->f_name, strerror(errno));
 				
 				}
 			} else {
@@ -213,7 +196,7 @@ entry_print_data(FILE *out, char n, struct r_entry *e)
 			}
 			/* hijack size for major,minor number when special */
 			if (S_ISBLK(e->f_mode) || S_ISCHR(e->f_mode)) {
-				fprintf(out, "%d,%d", major(e->f_rdev), minor(e->f_rdev));
+				fprintf(out, "%d,%d", (unsigned int) major(e->f_rdev),(unsigned int) minor(e->f_rdev));
 				break;
 			}
 
@@ -259,59 +242,33 @@ entry_print_data(FILE *out, char n, struct r_entry *e)
  * print function
  */
 void
-entry_print(FILE *out, char plusmin, struct r_entry *e)
+entry_print(FILE *out, guint pm, struct r_entry *e, char *fmt)
 {
 	char *pos;
-	struct stat s;
-
-	if ((plusmin == '+') && (opt_modified == FALSE)) {
+	if ((pm == PLUS) && (opt_modified == FALSE)) {
 		return;
 	}
 
-	if ((plusmin == '-') && (opt_removed == FALSE)) {
+	if ((pm == MINUS) && (opt_removed == FALSE)) {
 		return;
 	}
 
 	if (opt_verbose > 1) {
 		fputs("** ", stderr);
-		fputc(plusmin, stderr);
+		fputc(pm == PLUS ? '+' : '-', stderr);
 		fprintf(stderr, " %s\n", e->f_name);
 	}
 
-	if (S_ISREG(e->f_mode) && plusmin == '+' && !opt_local
-		&& e->f_lnk == 0) {
-		/* check if the file has changed since we first
-		 * visited it. If so, skip it as it will tear
-		 * up the entire print. Esp. when also printing
-		 * the contents. The recheck here, minimizes the
-		 * race, it's NOT GONE!!
-		 *
-		 * This is not a problem for directories, nor sym- and
-		 * hard links
-		 */
-		if (lstat(e->f_name, &s) != 0) {
-			msg(_("Could not stat path `%s\': %s"), e->f_name,
-				strerror(errno));
-			return;
-		}
-#ifndef _DEBUG_RACE
-		if (e->f_size != s.st_size) {
-			msg(_("File size has changed, skipping `%s\'"), e->f_name);
-			return;
-		}
-#endif /* _DEBUG_RACE */
-	}
-
 	/* next check if we can read the file, if not - skip it and don't emit
 	 * anything */
-	if (S_ISREG(e->f_mode) && plusmin == '+' && e->f_lnk == 0) {
+	if (S_ISREG(e->f_mode) && pm == PLUS && e->f_lnk == 0) {
 		if (access(e->f_name, R_OK) == -1) {
 			msg("Unable to open file `%s\': %s", e->f_name, strerror(errno));
 			return;
 		}
 	}
 
-	for (pos = opt_format; *pos != '\0';  ++pos) {
+	for (pos = fmt; *pos != '\0';  ++pos) {
 		switch (*pos) {
 			/* c-style escapes are valid */
 			case '\\':
@@ -326,10 +283,10 @@ entry_print(FILE *out, char plusmin, struct r_entry *e)
 						  fputc('%', out);
 						  break;
 					case 'p':
-						  fputc(plusmin, out);
+						  fputc(pm == PLUS ? '+' : '-', out);
 						  break;
 					case 'C':
-						  if (plusmin == '+') {
+						  if (pm == PLUS) {
 						  	entry_cat_data(out, e);
 						  }
 						  break;
@@ -364,8 +321,12 @@ gfunc_write(gpointer data, gpointer value, gpointer fp)
 	if (e->f_lnk == 1)
 		linktype = 'h';
 
-	fprintf((FILE*) fp, "%5ld %ld %ld %c %ld %ld %s", (long int)e->f_mode, (long int)e->f_dev, 
-			(long int)e->f_ino, linktype, (long int)strlen(e->f_name), (long int)e->f_size, 
+	fprintf((FILE*) fp, "%5ld %ld %ld %c %ld %ld %ld %ld %s", (long int)e->f_mode, 
+			(long int)e->f_dev, 
+			(long int)e->f_ino, linktype, 
+			(long int)e->f_uid, 
+			(long int)e->f_gid,
+			(long int)strlen(e->f_name), (long int)e->f_size, 
 			e->f_name);
 	if (opt_null) {
 		fputc('\0', (FILE*)fp);
@@ -381,6 +342,7 @@ gfunc_write(gpointer data, gpointer value, gpointer fp)
 gboolean
 gfunc_backup(gpointer data, gpointer value,
 		__attribute__((unused)) gpointer usr)
+
 {
 	if (sig != 0)
 		signal_abort(sig);
@@ -390,25 +352,21 @@ gfunc_backup(gpointer data, gpointer value,
 		return FALSE;
 
 	if (S_ISDIR(((struct r_entry*)data)->f_mode)) {
-		entry_print(stdout, '+', (struct r_entry*)data);
+		entry_print(stdout, PLUS, (struct r_entry*)data, opt_format);
 		return FALSE;
 	} else {
-		#if 0
-	if (S_ISREG(((struct entry*)data)->f_mode) ||
-			S_ISLNK(((struct entry*)data)->f_mode)) {
-		#endif
-
-		if (opt_size != 0 && ((struct r_entry*)data)->f_size > (ssize_t)opt_size) {
+		if (opt_size != 0 && S_ISREG(((struct r_entry*)data)->f_mode) &&
+			((struct r_entry*)data)->f_size > (ssize_t)opt_size) {
 			return FALSE;
 		}
 		switch (opt_timestamp) {
 			case NULL_DUMP:
-				entry_print(stdout, '+', (struct r_entry*)data);
+				entry_print(stdout, PLUS, (struct r_entry*)data, opt_format);
 				return FALSE;
 			default: /* INC_DUMP */
-				if (((struct r_entry*)data)->f_ctime > opt_timestamp)
-					entry_print(stdout, '+', (struct r_entry*)data);
-			
+				if (((struct r_entry*)data)->f_ctime >= opt_timestamp) {
+					entry_print(stdout, PLUS, (struct r_entry*)data, opt_format);
+				}
 				return FALSE;
 		}
 	}
@@ -419,8 +377,9 @@ gfunc_backup(gpointer data, gpointer value,
  * write out the list of removed items
  */
 gboolean
-gfunc_remove(gpointer data, gpointer value,
+gfunc_remove(gpointer data, gpointer value, 
 		__attribute__((unused)) gpointer usr)
+
 {
 	if (sig != 0)
 		signal_abort(sig);
@@ -430,8 +389,7 @@ gfunc_remove(gpointer data, gpointer value,
 		msg(_("Internal error: NO_PRINT in remove tree!"));
 		return FALSE;
 	}
-
-	entry_print(stdout, '-', (struct r_entry*)data);
+	entry_print(stdout, MINUS, (struct r_entry*)data, opt_format);
 	return FALSE;
 }
 
@@ -439,7 +397,7 @@ gfunc_remove(gpointer data, gpointer value,
  * Print out the list of new item
  */
 gboolean
-gfunc_new(gpointer data, __attribute__((unused)) gpointer value,
+gfunc_new(gpointer data, __attribute__((unused)) gpointer value, 
 		__attribute__((unused)) gpointer usr)
 {
 	if (sig != 0)
@@ -449,7 +407,12 @@ gfunc_new(gpointer data, __attribute__((unused)) gpointer value,
 	if (value == NO_PRINT) 
 		return FALSE;
 
-	entry_print(stdout, '+', (struct r_entry*)data);
+	if (opt_size != 0 && S_ISREG(((struct r_entry*)data)->f_mode) &&
+			((struct r_entry*)data)->f_size > (ssize_t)opt_size) {
+		return FALSE;
+	}
+
+	entry_print(stdout, PLUS, (struct r_entry*)data, opt_format);
 	return FALSE;
 }
 
@@ -461,17 +424,28 @@ gint
 gfunc_equal(gconstpointer a, gconstpointer b)
 {
 	gint e;
+	struct r_entry *ae, *be;
+
+	ae = (struct r_entry *)a;
+	be = (struct r_entry *)b;
 
 	if (sig != 0)
 		signal_abort(sig);
 
 	e = strcmp(((struct r_entry*)a)->f_name, ((struct r_entry*)b)->f_name);
 	if (e == 0) {
-		if (((struct r_entry*)a)->f_dev != ((struct r_entry*)b)->f_dev)
+		if (ae->f_dev != be->f_dev)
 			return -1;
-		if (((struct r_entry*)a)->f_ino != ((struct r_entry*)b)->f_ino) 
+		if (ae->f_ino != be->f_ino) 
 			return -2;
-		if (((struct r_entry*)a)->f_mode != ((struct r_entry*)b)->f_mode) 
+
+		/* if we are looking at a directory and only the mode has changed
+		 * don't let rdup remove the entire directory */
+		if (S_ISDIR(ae->f_mode) && S_ISDIR(be->f_mode) && 
+				((ae->f_mode & ~S_IFMT) != (be->f_mode & ~S_IFMT)) )
+			return 0;
+
+		if (ae->f_mode != be->f_mode) 
 			return -3;
 	}
 	return e;
