@@ -5,6 +5,8 @@
 
 #include "rdup-up.h"
 
+extern char *PROGNAME;
+
 void
 usage_up(FILE *f)
 {
@@ -18,11 +20,12 @@ The input must be rdup -c output\n\
 \n\
     OPTIONS:\n\
 	-t\t\tcreate DIRECTORY if is does not exist\n\
-	-s N\t\tstrip N leading path components from each path\n\
+	-s NUM\t\tstrip NUM leading path components\n\
+	-r PATH\t\tstrip PATH from each pathname\n\
 	-n\t\tdry run, do not touch the filesystem\n\
 	-V\t\tprint version\n\
 	-h\t\tthis help\n\
-	-v\t\tbe more verbose and print processed files to stderr\n\
+	-v\t\tbe more verbose and print processed files to stdout\n\
 \n\
 Report bugs to <miek@miek.nl>\n\
 Licensed under the GPL version 3.\n\
