@@ -5,6 +5,9 @@
  * Not used anymore - this is a hack that I don't want to
  * support anymore. It stinks
  *
+ * It was used in saving the uid/gid information when doing
+ * a remote dump as a no root user.
+ *
  * All xattr functions are grouped here
  */
 
@@ -85,7 +88,7 @@ read_attr_gid(__attribute__((unused))
 		return x;
 	} else {
 		if (opt_verbose > 0) {
-			msg(_("No gid xattr for `%s\'\n"), path);
+			msg(_("No gid xattr for `%s\'"), path);
 		}
 		return g;
 	}
